package com.zh.lingnuo.aop;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.zh.lingnuo.annotation.Token;
import com.zh.lingnuo.model.*;
import com.zh.lingnuo.service.*;
import com.zh.lingnuo.utils.BizException;
import com.zh.lingnuo.utils.Common;
import com.zh.lingnuo.utils.result.HttpStatusEnum;
import org.apache.commons.lang3.StringUtils;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;


/**
 * 检查权限切面类
 */
@Order(2)
@Aspect
@Component
public class TokenAspect {

    @Autowired
    private IUserTokenService userTokenService;
    @Autowired
    private IUserService userService;
    @Autowired
    private ICompanyService companyService;
    @Autowired
    private IAnswererService answererService;

    /**
     * 功能描述: 切点
     */
    @Pointcut("@annotation(com.zh.lingnuo.annotation.Token)")
    public void executePointCut() {

    }

    /**
     * 功能描述: 请求前执行
     */
    @Before("executePointCut()")
    public void doBefore(JoinPoint joinPoint) {
        // 接收到请求，记录请求内容
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        Object[] args = joinPoint.getArgs();
        //获取当前用户的信息
        String token = request.getHeader("token");
        if (!StringUtils.isNotBlank(token)) {
            throw new BizException(HttpStatusEnum.LOGIN_FAILED);
        }
        UserToken userToken = userTokenService.getOne(new QueryWrapper<UserToken>().eq("token", token).last("limit 1"));
        if (null == userToken) {
            throw new BizException(HttpStatusEnum.LOGIN_FAILED);
        }
        MethodSignature signature = (MethodSignature) joinPoint.getSignature();
        int level = userToken.getKind();
        int[] levels = signature.getMethod().getAnnotation(Token.class).level();

        if (!Common.isContain(levels, level)) {
            throw new BizException(HttpStatusEnum.PERMISSION_DENIED);
        }
        int requireUser = signature.getMethod().getAnnotation(Token.class).requireUser();
        if(1 != requireUser){
            return;
        }
        Integer userId;
        switch (level) {
            case 1:
                User user = userService.getOne(new QueryWrapper<User>().eq("id", userToken.getUserId()).last("limit 1"));
                if (null == user) {
                    throw new BizException(HttpStatusEnum.USER_NOT_EXIST);
                }
                userId = user.getId();
                Common.changObjectValue(args[0], "userInfo", user);
                break;
            case 2:
                Company company = companyService.getOne(new QueryWrapper<Company>().eq("id", userToken.getUserId()).last("limit 1"));
                if (null == company) {
                    throw new BizException(HttpStatusEnum.USER_NOT_EXIST);
                }
                if (Company.ManageCompanyStatusForbidden == company.getStatus()) {
                    throw new BizException(HttpStatusEnum.PERMISSION_DENIED);
                }
                userId = company.getId();
                Common.changObjectValue(args[0], "userInfo", company);
                break;
            case 3:
                Answerer answerer = answererService.getOne(new QueryWrapper<Answerer>().eq("id", userToken.getUserId()).last("limit 1"));
                if (null == answerer) {
                    throw new BizException(HttpStatusEnum.USER_NOT_EXIST);
                }
                userId = answerer.getId();
                Common.changObjectValue(args[0], "userInfo", answerer);
                break;
            default:
                throw new BizException(HttpStatusEnum.USER_NOT_EXIST);
        }
        if (!Common.higherZero(userId)) {
            throw new BizException(HttpStatusEnum.USER_NOT_EXIST);
        }
    }

    /**
     * 请求后执行
     */
    @AfterReturning(returning = "ret", pointcut = "executePointCut()")
    public void doAfterReturning(Object ret) {

    }
}
