package com.zh.lingnuo.controller.manage;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.zh.lingnuo.annotation.Token;
import com.zh.lingnuo.controller.company.CompanyQnController;
import com.zh.lingnuo.controller.company.CompanyQnTitleController;
import com.zh.lingnuo.model.Company;
import com.zh.lingnuo.model.CompanyQn;
import com.zh.lingnuo.model.CompanyQnTitle;
import com.zh.lingnuo.service.ICompanyQnAnswerService;
import com.zh.lingnuo.service.ICompanyQnService;
import com.zh.lingnuo.service.ICompanyQnTitleService;
import com.zh.lingnuo.utils.PageUtil;
import com.zh.lingnuo.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import lombok.experimental.Accessors;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.net.URLEncoder;
import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-09-13
 */
@Tag(name = "管理企业问卷")
@RestController
@RequestMapping("/manage_company_qn")
public class ManageCompanyQnController {

    @Autowired
    private ICompanyQnService companyQnService;
    @Autowired
    private ICompanyQnTitleService companyQnTitleService;
    @Autowired
    private ICompanyQnAnswerService companyQnAnswerService;

    @Data
    public static class paramsManageCompanyQnList {
        private int pageNum;
        private int pageSize;
        @Schema(title = "名称")
        private String title;
        @Schema(title = "公司名称")
        private String companyName;
        @Schema(title = "状态")
        private Integer status;
        @Schema(title = "更新时间起")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime updateTimeStart;
        @Schema(title = "更新时间止")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime updateTimeEnd;
        @Schema(title = "创建时间起")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime createTimeStart;
        @Schema(title = "创建时间止")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime createTimeEnd;
    }



    @Operation(summary = "列表")
    @PostMapping("/list")
    @Token(requireUser = 1, level = {1})
    public Result<IPage<CompanyQn>> list(@Validated @RequestBody paramsManageCompanyQnList params) {
        return companyQnService.manageQnList(params);
    }


    @Data
    public static class ParamsManageCompanyQnOpr {
        @Schema(title = "qnId")
        @NotNull(message = "qnId不能为空")
        private Integer qnId;
    }

    @Operation(summary = "问卷题目列表")
    @PostMapping("/titleList")
    @Token(requireUser = 1, level = {1})
    public Result<List<CompanyQnTitleController.ResultQnTitleList>> titleList(@Validated @RequestBody ParamsManageCompanyQnOpr params) {

        return companyQnTitleService.qnTitleList(params.getQnId());
    }

    @Operation(summary = "问卷结果")
    @PostMapping("/result")
    @Token(requireUser = 1, level = {1})
    public Result<CompanyQnController.ResultCompanyQn> qnResult(@Validated @RequestBody ParamsManageCompanyQnOpr params) {
        return companyQnService.result(params.getQnId());
    }

    @Operation(summary = "答题人互评详情")
    @PostMapping("/partnerChoiceResult")
    @Token(requireUser = 1, level = {1})
    public Result partnerChoiceResult(@Validated @RequestBody CompanyQnController.paramsPartnerChoiceResult params) {
        return companyQnAnswerService.partnerChoiceResult(PageUtil.SetPage(params.getPageNum(),params.getPageSize()),params.getQnId(),true);
    }

    @Operation(summary = "答题人互评结果导出Excel")
    @PostMapping("/resultExportExcel")
    @Token(requireUser = 1, level = {1})
    public void resultExportExcel(HttpServletResponse response, @Validated @RequestBody CompanyQnController.paramsPartnerChoiceResult params) {
        companyQnAnswerService.resultExportExcel(response,params.getQnId(),true);
    }

}
