package com.zh.lingnuo.controller.manage;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.zh.lingnuo.annotation.Token;
import com.zh.lingnuo.model.QnGroup;
import com.zh.lingnuo.model.QnTitle;
import com.zh.lingnuo.service.IQnGroupService;
import com.zh.lingnuo.service.IQnTitleService;
import com.zh.lingnuo.utils.result.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author zh
 * @since 2022-09-13
 */
@Tag(name = "分类管理")
@RestController
@RequestMapping("/qn_group")
public class QnGroupController {

    @Autowired
    private IQnGroupService qnGroupService;

    @Autowired
    private IQnTitleService qnTitleService;

    @Data
    public static class paramsGroupList {
        @Schema(title = "名称")
        private String title;
    }

    @Operation(summary = "列表")
    @PostMapping("/list")
    @Token(requireUser=1,level = {1})
    public Result<List<QnGroup>> list(@Validated @RequestBody paramsGroupList params) {
        return qnGroupService.groupList(params);
    }

    @Data
    public static class paramsGroupAdd {
        @Schema(title = "名称")
        @NotNull(message = "名称不能为空")
        @NotEmpty(message = "名称不能为空")
        private String title;
    }

    @Operation(summary = "新增")
    @PostMapping("/add")
    @Token(requireUser=1,level = {1})
    public Result add(@Validated @RequestBody paramsGroupAdd params) {
        if (!qnGroupService.save(new QnGroup()
            .setTitle(params.getTitle())
            .setStatus(1))) {
            return Result.err("错误");
        }
        return Result.success("");
    }

    @Data
    public static class paramsGroupUpd {
        @Schema(title = "id")
        @NotNull(message = "id不能为空")
        private Integer id;
        @Schema(title = "名称")
        @NotNull(message = "名称不能为空")
        @NotEmpty(message = "名称不能为空")
        private String title;
    }

    @Operation(summary = "修改")
    @PostMapping("/upd")
    @Token(requireUser=1,level = {1})
    public Result upd(@Validated @RequestBody paramsGroupUpd params) {
        if(params.getId() == 1){
            return Result.err("基础信息不可更改");
        }
        if (!qnGroupService.update(new QnGroup(), new UpdateWrapper<QnGroup>()
            .eq("id", params.getId())
            .set("title", params.getTitle()))) {
            return Result.err("错误");
        }
        return Result.success("");
    }

    @Data
    public static class paramsGroupDel {
        @Schema(title = "id")
        @NotNull(message = "id不能为空")
        private Integer id;
    }

    @Operation(summary = "删除")
    @PostMapping("/del")
    @Token(requireUser=1,level = {1})
    public Result del(@Validated @RequestBody paramsGroupDel params) {
        if(params.getId() == 1){
            return Result.err("基础信息不可删除");
        }
        if (!qnGroupService.removeById(params.getId())) {
            return Result.err("错误");
        }
        return Result.success("");
    }

    @Data
    public static class paramsGroupAddTitle {
        @Schema(title = "id")
        @NotNull(message = "id不能为空")
        private Integer id;
        @Schema(title = "题目id")
        @NotNull(message = "题目id数组不能为空")
        @NotEmpty(message = "题目id数组不能为空")
        private List<Integer> titleIds;
    }

    @Operation(summary = "添入题目")
    @PostMapping("/addTitle")
    @Token(requireUser=1,level = {1})
    public Result addTitle(@Validated @RequestBody paramsGroupAddTitle params) {
        if (null == qnGroupService.getById(params.getId())) {
            return Result.err("未知分组");
        }
        if (!qnTitleService.update(new QnTitle(),
            new UpdateWrapper<QnTitle>()
                .in("id", params.getTitleIds())
                .set("group_id", params.getId()))) {
            return Result.err("无效操作");
        }
        return Result.success("");
    }

}
