package com.zh.lingnuo.model;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.baomidou.mybatisplus.annotation.TableId;
import java.time.LocalDateTime;

import com.zh.lingnuo.utils.Common;
import com.zh.lingnuo.utils.FuncRes;
import com.zh.lingnuo.utils.result.HttpStatusEnum;
import com.zh.lingnuo.utils.result.Result;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

/**
 * <p>
 *
 * </p>
 *
 * @author zh
 * @since 2022-09-13
 */
@Data
@EqualsAndHashCode(callSuper = true)
@Accessors(chain = true)
public class QnTitle extends Model {

    private static final long serialVersionUID = 1L;

    @TableId(value = "id", type = IdType.AUTO)
    private Integer id;

    /**
     * 题目
     */
    private String title;

    /**
     * 1=填空题 2=选择题 3=互选题
     */
    private Integer kind;

    /**
     * 0=不可用 1 =可用
     */
    private Integer status;

    /**
     * 分类id
     */
    private Integer groupId;

    /**
     * 标签
     */
    private String tag;

    /**
     * 选项
     */
    private String options;

    @TableField(fill = FieldFill.INSERT_UPDATE)
    private LocalDateTime updatedAt;

    @TableField(fill = FieldFill.INSERT)
    private LocalDateTime createdAt;

    /**  填空题 */
    public static int QnTitleKindFill = 1;
    /**  选择题 */
    public static int QnTitleKindChoice = 2;
    /**  互选题 */
    public static int QnTitleKindPartnerChoice = 3;
    /**  多选题 */
    public static int QnTitleKindMultiChoice = 4;

    public static String getKindName(int kind) {
        switch (kind) {
            case 1:
                return "填空题";
            case 2:
                return "选择题";
            case 3:
                return "互选题";
            case 4:
                return "多选题";
            default:
                return "未知";
        }
    }

    public static String getStatusName(int status) {
        switch (status) {
            case 0:
                return "不可用";
            case 1:
                return "可用";
            default:
                return "未知";
        }
    }

    public static FuncRes optionsToObj(String []options){
            if (null == options || options.length <= 0) {
                return FuncRes.fail("请补充选项");
            }
            if (options.length > CompanyQnTitle.ManageQnTitleMaxOptions) {
                return FuncRes.fail("不支持过多的选项");
            }
            String optionsStr = arrToObj(options).toString();
            return FuncRes.success(optionsStr,"");
    }

    // 数组保存为obj
    private static Object arrToObj(String[] arr) {
        JSONObject obj = new JSONObject();
        if (null == arr || arr.length <= 0 || arr.length > 26) {
            return obj;
        }
        String str = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
        int i = 0;
        for (String s : arr) {
            obj.put(String.valueOf(str.charAt(i)), s);
            i++;
        }
        return obj;
    }

}
