package com.zh.lingnuo.serviceImpl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.zh.lingnuo.controller.manage.QnTitleController;
import com.zh.lingnuo.model.CompanyQnTitle;
import com.zh.lingnuo.model.QnTitle;
import com.zh.lingnuo.mapper.QnTitleMapper;
import com.zh.lingnuo.service.IQnGroupService;
import com.zh.lingnuo.service.IQnTitleService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zh.lingnuo.utils.Common;
import com.zh.lingnuo.utils.FuncRes;
import com.zh.lingnuo.utils.PageUtil;
import com.zh.lingnuo.utils.result.Result;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author zh
 * @since 2022-09-13
 */
@Service
public class QnTitleServiceImpl extends ServiceImpl<QnTitleMapper, QnTitle> implements IQnTitleService {

    @Autowired
    private IQnGroupService qnGroupService;

    @Autowired
    private QnTitleMapper qnTitleMapper;


    // 列表
    public Result<IPage<QnTitleController.resultTitleList>> titleList(QnTitleController.paramsTitleList params) {
        QueryWrapper<QnTitle> qw = new QueryWrapper<>();

        qw.like(StringUtils.isNotBlank(params.getTitle()), "t1.title", params.getTitle())
            .ge(null != params.getCreateTimeStart(), "t1.created_at", params.getCreateTimeStart())
            .le(null != params.getCreateTimeEnd(), "t1.created_at", params.getCreateTimeEnd())
            .ge(null != params.getUpdateTimeStart(), "t1.updated_at", params.getUpdateTimeStart())
            .le(null != params.getUpdateTimeEnd(), "t1.updated_at", params.getUpdateTimeEnd())
        .orderByDesc("t1.id");
        {
            String groupWhere = "select id from qn_group where";
            if (StringUtils.isNotBlank(params.getGroupTitle())) {
                groupWhere += " title like " + "'%" + params.getGroupTitle() + "%'";
            }
            if (null != params.getGroupId()) {
                groupWhere += " id = " + params.getGroupId();
            }
            if (!groupWhere.substring(groupWhere.length() - 5).equals("where")) {
                qw.inSql("group_id", groupWhere);
            }
        }

        IPage page = qnTitleMapper.list(PageUtil.SetPage(params.getPageNum(), params.getPageSize()), qw);
        page.convert(r -> {
            QnTitleController.resultTitleList v = new QnTitleController.resultTitleList();
            BeanUtils.copyProperties(r, v);
            v.setGroupId((Integer) Common.ObjectFieldValue(r, "gpId"));
            v.setOptions(JSONObject.parseObject((String) Common.ObjectFieldValue(r, "options")));
            v.setKindName(QnTitle.getKindName((Integer) Common.ObjectFieldValue(r, "kind")));
            v.setStatusName(QnTitle.getStatusName((Integer) Common.ObjectFieldValue(r, "status")));
            return v;
        });
        return Result.success(page);
    }

    // 添加
    public Result titleAdd(QnTitleController.paramsTitleAdd params) {
        if (null == qnGroupService.getById(params.getGroupId())) {
            return Result.err("未知分组");
        }
        QnTitle qnTitle = new QnTitle()
            .setTitle(params.getTitle())
            .setGroupId(params.getGroupId())
            .setTag(params.getTag())
            .setKind(params.getKind())
            .setStatus(1);
        {
            String[] options = params.getOptions();
            if (Common.isContain(new int[]{QnTitle.QnTitleKindChoice, QnTitle.QnTitleKindPartnerChoice,QnTitle.QnTitleKindMultiChoice}, params.getKind())) {
                FuncRes funcRes = QnTitle.optionsToObj(options);
                if (!funcRes.isSuccess()) {
                    return Result.err(funcRes.getMsg());
                }
                qnTitle.setOptions((String) funcRes.getData());
            }
        }
        if (!super.save(qnTitle)) {
            return Result.err("添加出错");
        }
        return Result.success("");
    }

    // 修改
    public Result titleUpd(QnTitleController.paramsTitleUpd params) {
        UpdateWrapper<QnTitle> qw = new UpdateWrapper<QnTitle>().eq("id", params.getId());
        if (Common.higherZero(params.getGroupId())) {
            if (null == qnGroupService.getById(params.getGroupId())) {
                return Result.err("未知分组");
            }
            qw.set("group_id", params.getGroupId());
        }
        {
            String[] options = params.getOptions();
            if (Common.isContain(new int[]{QnTitle.QnTitleKindChoice, QnTitle.QnTitleKindPartnerChoice,QnTitle.QnTitleKindMultiChoice}, params.getKind())) {
                FuncRes funcRes = QnTitle.optionsToObj(options);
                if (!funcRes.isSuccess()) {
                    return Result.err(funcRes.getMsg());
                }
                qw.set("options",(String) funcRes.getData());
            }
        }
        if (!super.update(new QnTitle(), qw
            .set(StringUtils.isNotBlank(params.getTitle()), "title", params.getTitle())
            .set(StringUtils.isNotBlank(params.getTag()),"tag",params.getTag())
            .set(Common.higherZero(params.getKind()), "kind", params.getKind()))) {
            return Result.err("操作失败");
        }
        return Result.success("");
    }
}
