package com.zh.lingnuo.utils.result;

/**
 * http所有状态码枚举
 */
public enum HttpStatusEnum {
    /**
     * http状态码枚举所有状态码注解
     */
    CONTINUE(100, "Continue", "请继续发送请求的剩余部分"),
    SWITCHING_PROTOCOLS(101, "Switching Protocols", "协议切换"),
    PROCESSING(102, "Processing", "请求将继续执行"),
    CHECKPOINT(103, "Checkpoint", "可以预加载"),
    OK(200, "OK", "请求已经成功处理"),
    CREATED(201, "Created", "请求已经成功处理，并创建了资源"),
    ACCEPTED(202, "Accepted", "请求已经接受，等待执行"),
    NON_AUTHORITATIVE_INFORMATION(203, "Non-Authoritative Information", "请求已经成功处理，但是信息不是原始的"),
    NO_CONTENT(204, "No Content", "请求已经成功处理，没有内容需要返回"),
    RESET_CONTENT(205, "Reset Content", "请求已经成功处理，请重置视图"),
    PARTIAL_CONTENT(206, "Partial Content", "部分Get请求已经成功处理"),
    MULTI_STATUS(207, "Multi-Status", "请求已经成功处理，将返回XML消息体"),
    ALREADY_REPORTED(208, "Already Reported", "请求已经成功处理，一个DAV的绑定成员被前一个请求枚举，并且没有被再一次包括"),
    IM_USED(226, "IM Used", "请求已经成功处理，将响应一个或者多个实例"),
    MULTIPLE_CHOICES(300, "Multiple Choices", "提供可供选择的回馈"),
    MOVED_PERMANENTLY(301, "Moved Permanently", "请求的资源已经永久转移"),
    FOUND(302, "Found", "请重新发送请求"),
    // MOVED_TEMPORARILY(302, "Moved Temporarily", "") 已经过时
    SEE_OTHER(303, "See Other", "请以Get方式请求另一个URI"),
    NOT_MODIFIED(304, "Not Modified", "资源未改变"),
    USE_PROXY(305, "Use Proxy", "请通过Location域中的代理进行访问"),
    // 306在新版本的规范中被弃用
    TEMPORARY_REDIRECT(307, "Temporary Redirect", "请求的资源临时从不同的URI响应请求"),
    RESUME_INCOMPLETE(308, "Resume Incomplete", "请求的资源已经永久转移"),
    BAD_REQUEST(400, "Bad Request", "请求错误，请修正请求"),
    UNAUTHORIZED(401, "Unauthorized", "没有被授权或者授权已经失效"),
    PAYMENT_REQUIRED(402, "Payment Required", "预留状态"),
    FORBIDDEN(403, "Forbidden", "请求被理解，但是拒绝执行"),
    NOT_FOUND(404, "Not Found", "资源未找到"),
    METHOD_NOT_ALLOWED(405, "Method Not Allowed", "请求方法不允许被执行"),
    NOT_ACCEPTABLE(406, "Not Acceptable", "请求的资源不满足请求者要求"),
    PROXY_AUTHENTICATION_REQUIRED(407, "Proxy Authentication Required", "请通过代理进行身份验证"),
    REQUEST_TIMEOUT(408, "Request Timeout", "请求超时"),
    CONFLICT(409, "Conflict", "请求冲突"),
    GONE(410, "Gone", "请求的资源不可用"),
    LENGTH_REQUIRED(411, "Length Required", "Content-Length未定义"),
    PRECONDITION_FAILED(412, "Precondition Failed", "不满足请求的先决条件"),
    REQUEST_ENTITY_TOO_LARGE(413, "Request Entity Too Large", "请求发送的实体太大"),
    REQUEST_URI_TOO_LONG(414, "Request-URI Too Long", "请求的URI超长"),
    UNSUPPORTED_MEDIA_TYPE(415, "Unsupported Media Type", "请求发送的实体类型不受支持"),
    REQUESTED_RANGE_NOT_SATISFIABLE(416, "Requested range not satisfiable", "Range指定的范围与当前资源可用范围不一致"),
    EXPECTATION_FAILED(417, "Expectation Failed", "请求头Expect中指定的预期内容无法被服务器满足"),
    // I_AM_A_TEAPOT(418, "I'm a teapot", ""), 该代码没有被服务器实现

    // INSUFFICIENT_SPACE_ON_RESOURCE(419, "Insufficient Space On Resource", ""),  已经过时

    // METHOD_FAILURE(420, "Method Failure", ""),  已经过时

    // DESTINATION_LOCKED(421, "Destination Locked", ""),  已经过时
    UNPROCESSABLE_ENTITY(422, "Unprocessable Entity", "请求格式正确，但是由于含有语义错误，无法响应"),
    LOCKED(423, "Locked", "当前资源被锁定"),
    FAILED_DEPENDENCY(424, "Failed Dependency", "由于之前的请求发生错误，导致当前请求失败"),
    UPGRADE_REQUIRED(426, "Upgrade Required", "客户端需要切换到TLS1.0"),
    PRECONDITION_REQUIRED(428, "Precondition Required", "请求需要提供前置条件"),
    TOO_MANY_REQUESTS(429, "Too Many Requests", "请求过多"),
    REQUEST_HEADER_FIELDS_TOO_LARGE(431, "Request Header Fields Too Large", "请求头超大，拒绝请求"),
    INTERNAL_SERVER_ERROR(500, "Internal Server Error", "服务器内部错误"),
    NOT_IMPLEMENTED(501, "Not Implemented", "服务器不支持当前请求的部分功能"),
    BAD_GATEWAY(502, "Bad Gateway", "响应无效"),
    SERVICE_UNAVAILABLE(503, "Service Unavailable", "服务器维护或者过载，拒绝服务"),
    GATEWAY_TIMEOUT(504, "Gateway Timeout", "上游服务器超时"),
    HTTP_VERSION_NOT_SUPPORTED(505, "HTTP Version not supported", "不支持的HTTP版本"),
    VARIANT_ALSO_NEGOTIATES(506, "Variant Also Negotiates", "服务器内部配置错误"),
    INSUFFICIENT_STORAGE(507, "Insufficient Storage", "服务器无法完成存储请求所需的内容"),
    LOOP_DETECTED(508, "Loop Detected", "服务器处理请求时发现死循环"),
    BANDWIDTH_LIMIT_EXCEEDED(509, "Bandwidth Limit Exceeded", "服务器达到带宽限制"),
    NOT_EXTENDED(510, "Not Extended", "获取资源所需的策略没有被满足"),
    NETWORK_AUTHENTICATION_REQUIRED(511, "Network Authentication Required", "需要进行网络授权"),



    //业务错误码
    CUSTOM_EXCEPTION(1001, "Custom exception", "自定义异常"),
    ACCOUNT_PASSWORD_ERROR(10002, "Account or password Error", "账户或密码错误"),
    TEL_CODE_ERROR(10003, "Tel Code Error", "手机验证码有误"),
    SEND_TEL_CODE_ERROR(10004, "Send Tel Code Error", "发送手机验证码失败"),
    VERIFICATION_CODE_FAILURE(10005, "Verification Code Failure", "验证码失效"),
    VERIFICATION_CODE_EXPIRED(10006, "Verification Code Expired", "验证码已过期"),
    VERIFICATION_CODE_ERROR(10007, "Verification Code Error", "验证码错误"),
    USER_NOT_EXIST(10008, "User Not Exist", "用户不存在"),
    USER_IS_DELETE(10009, "The user has been deleted, please find the administrator to restore", "用户已被删除，请找管理员恢复"),
    LOGIN_FAILED(10010, "Login failed, please login again", "登录失效，请重新登录"),
    TEL_REGISTERED(10011, "Mobile number registered", "手机号已注册"),
    REGISTER_FAILED(10012, "register has failed", "注册失败"),
    RECORD_NOT_EXIST(10013,"The current record does not exist","当前记录不存在"),
    OPERATION_FAILED(10014,"operation failed","操作失败"),
    FILE_NULL(10015,"file is empty","文件为空"),
    FILE_TOO_LARGE(10016,"The file size cannot be greater than 10m","文件大小不能大于10M"),
    FILE_IMG_FORMAT_ERROR(10017,"Please select pictures in JPG, JPEG, GIF, PNG, PDF format","请选择jpg,jpeg,gif,png,pdf格式的图片"),
    FILE_DOC_FORMAT_ERROR(10018,"Please select a file in Doc, docx format","请选择doc,docx格式的文件"),
    FILE_XLS_FORMAT_ERROR(10019,"Please select excel in XLS, xlsx format","请选择xls,xlsx格式的excel"),
    FILE_SAVE_EXCEPTION(10020,"Save file exception","保存文件异常"),
    FILE_SAVE_DATA_FAIL(10021,"Save file Data fail","添加照片数据失败"),
    FILE_DELETE_DATA_FAIL(10022,"Delete file Data fail","数据库资源删除失败"),
    FILE_DELETE_FAIL(10023,"Delete file fail","文件删除失败"),
    FILE_UPLOAD_AGAIN(10024,"The photo information is incorrect. Please upload it again","照片信息有误，请重新上传"),
    COMPANY_INFORMATION_ENTERED(10025,"Company information entered","公司信息已录入"),
    CHARGING_STANDARD_NOT_ENABLED(10026,"Charging standard is not enabled","收费标准未开启"),
    MISSING_FIELD_INFO(10027,"Missing field information","缺少字段信息"),
    COMPANY_NOT_BOUND(10028,"Company not bound","未绑定公司信息"),
    PRICE_INCONSISTENCY(10028,"Price inconsistency","价格不一致"),
    BOUND_CELL_ALREADY_EXISTS(10029,"Bound cell already exists","已存在绑定小区"),
    ALIPAY_REFUND_FAILED(10030,"Alipay refund failed","支付宝退款失败"),
    NOT_CHOOSE_PAY_TYPE(10031,"No payment method selected","未选择支付方式"),
    WRONG_CELL_ID(10032,"Wrong cell ID","小区识别码有误"),
    PERMISSION_DENIED(10033,"Permission denied","权限不足"),
    NOT_EXIST_ASSOCIATED_RECORD(10034,"Cannot be deleted. There are associated records","不可删除，存在关联记录"),
    MOBILE_PHONE_NUMBER_FORMAT_ERROR(10034,"Mobile phone number format error","手机号格式错误");


    private final int code;
    private final String reasonPhraseUS;
    private final String reasonPhraseCN;

    private static final int
        INFORMATIONAL = 1,
        SUCCESSFUL = 2,
        REDIRECTION = 3,
        CLIENT_ERROR = 4,
        SERVER_ERROR = 5;

    HttpStatusEnum(int code, String reasonPhraseUS, String reasonPhraseCN) {
        this.code = code;
        this.reasonPhraseUS = reasonPhraseUS;
        this.reasonPhraseCN = reasonPhraseCN;
    }

    public int code() {
        return code;
    }

    public String reasonPhraseUS() {
        return reasonPhraseUS;
    }

    public String reasonPhraseCN() {
        return reasonPhraseCN;
    }

    public static HttpStatusEnum valueOf(int code) {
        for (HttpStatusEnum httpStatus : values()) {
            if (httpStatus.code() == code) {
                return httpStatus;
            }
        }
        throw new IllegalArgumentException("No matching constant for [" + code + "]");
    }

    public boolean is1xxInformational() {
        return type() == INFORMATIONAL;
    }

    public boolean is2xxSuccessful() {
        return type() == SUCCESSFUL;
    }

    public boolean is3xxRedirection() {
        return type() == REDIRECTION;
    }

    public boolean is4xxClientError() {
        return type() == CLIENT_ERROR;
    }

    public boolean is5xxServerError() {
        return type() == SERVER_ERROR;
    }

    private int type() {
        return (int) code / 100;
    }
}
